import React, { useEffect, useState } from 'react'
import { yupResolver } from '@hookform/resolvers/yup'
import { useRecoilState } from 'recoil'
import { messageAtom } from 'src/lib/atoms/messageAtom'
import * as yup from 'yup'
import useSWR from 'swr'
import useErrors from 'src/hooks/useErrors'
import useAxios from 'src/hooks/useAxios'
import { useForm } from 'react-hook-form'
import { SaveIcon } from '@heroicons/react/solid'
import {
  Button,
  Checkbox,
  FormControlLabel,
  CircularProgress,
  Dialog,
  DialogActions,
  DialogContent,
  DialogContentText,
  DialogTitle,
  FormControl,
  FormHelperText,
  InputLabel,
  MenuItem,
  Select,
  TextField
} from '@mui/material'
import Box from '@mui/material/Box'

const ITEM_HEIGHT = 48
const ITEM_PADDING_TOP = 8

const MenuProps = {
  PaperProps: {
    style: {
      maxHeight: ITEM_HEIGHT * 4.5 + ITEM_PADDING_TOP,
      width: 250
    }
  }
}

const UserForm = ({ onClose, onSuccess, setOpenForm, userId }) => {
  const [open, setOpen] = useState(true) // This state is used for Opening the Form
  const [loading, setLoading] = useState(false) // This state is used for setting Loader
  const [buttonLoading, setButtonLoading] = useState(false) // This state is used for setting the Button Click loader
  const [message, setMessage] = useRecoilState(messageAtom)
  const [roleId, setRoleId] = useState(0) // This state is used for setting Role Value
  const { validationErrors } = useErrors() // Validating the Errors that come from the response
  const request = useAxios()
  const { errorHandler } = useErrors()

  // Once we finish the INSERT or UPDATE task we call this method to close the form
  // And change the states
  const handleClose = () => {
    setOpen(false)
    onClose(false)
    setOpenForm(false)
  }

  // The schema for the inputs that will be submitted
  const schema = yup.object().shape({
    first_name: yup.string().required(),
    last_name: yup.string().required(),
    email: yup.string().email().required(),
    role_id: yup.number().required()
  })

  // Required variables for using the useForm hook
  const {
    handleSubmit,
    register,
    formState: { errors },
    setError,
    setValue
  } = useForm({
    resolver: yupResolver(schema)
  })

  // Handling the loader if the userId is set
  useEffect(() => {
    if (userId !== 0) {
      setLoading(true)
    }
  }, [userId])

  // Handling the submit functionality of the form and checking the type of the method
  // to be send to the server
  const onSubmit = data => {
    if (userId !== 0) {
      data = { ...data, user_id: userId }
    }

    setButtonLoading(true)
    let url = userId === 0 ? '/users' : `users/${userId}`
    let method = userId === 0 ? 'POST' : 'PUT'

    request({
      url: url,
      method: method,
      data: data
    })
      .then(res => {
        if (res?.data?.status) {
          setButtonLoading(false)
          setMessage(res?.data?.message)
          onSuccess(true)
          handleClose()
        }
      })
      .catch(err => {
        setButtonLoading(false)
        validationErrors(err, setError)
      })
  }

  // Retriving the data respected to the userId
  const { data } = useSWR(`get/users/${userId}`, () =>
    request
      .get(`/users/${userId}`)
      .then(res => res.data)
      .catch(err => handleError(err))
  )

  // Retrive Roles data
  const { data: roles } = useSWR(`get/roles`, () =>
    request
      .get(`/roles?all`)
      .then(res => res.data)
      .catch(err => handleError(err))
  )

  // Setting fields value respected to the retrieved data from the request
  // If the userId is greater then 0 and data is not empty
  useEffect(() => {
    if (userId !== 0 && data) {
      data = data?.data
      setValue('first_name', data?.first_name)
      setValue('last_name', data?.last_name)
      setValue('email', data?.email)
      setValue('role_id', data?.role?.id)
      setRoleId(data?.role?.id)
      setLoading(false)
    }
  }, [data, userId])

  // Return the render content
  return (
    <div>
      <Dialog
        open={open}
        keepMounted
        onClose={handleClose}
        aria-describedby={'alert-dialog-slide-description'}
        maxWidth={'md'}
        fullWidth={true}
      >
        <form onSubmit={handleSubmit(onSubmit)}>
          <DialogTitle>{userId !== 0 ? 'Edit User' : 'Add USer'}</DialogTitle>
          <DialogContent>
            <DialogContentText className='py-2' component={'div'}>
              {loading ? (
                <div className={'py-8 text-center'}>
                  <CircularProgress size={40} />
                </div>
              ) : (
                <Box
                  sx={{
                    '& .MuiTextField-root': {
                      mt: 3,
                      mb: 2,
                      pr: 3
                    },
                    '& .MuiFormControl-root': {
                      mt: 3,
                      mb: 2,
                      pr: 3
                    },
                    display: 'flex',
                    flexDirection: 'row',
                    flexWrap: 'wrap'
                  }}
                >
                  <TextField
                    id='first_name'
                    size='small'
                    fullWidth
                    {...register('first_name')}
                    sx={{ width: 1 / 2 }}
                    label='First Name'
                    error={errors['first_name']}
                    helperText={errors['first_name']?.message}
                  />

                  <TextField
                    id='last_name'
                    size='small'
                    {...register('last_name')}
                    label='Last Name'
                    error={errors['last_name']}
                    helperText={errors['last_name']?.message}
                    fullWidth={true}
                    sx={{ width: 1 / 2 }}
                  />

                  <TextField
                    id='email'
                    size='small'
                    {...register('email')}
                    label='Email Address'
                    error={errors['email']}
                    helperText={errors['email']?.message}
                    fullWidth={true}
                    sx={{ width: 1 / 2 }}
                  />

                  <FormControl size='small' sx={{ width: 1 / 2 }} error={errors['role_id']} fullWidth={true}>
                    <InputLabel id='roles-label'>Roles</InputLabel>
                    {roles ? (
                      <Select
                        labelId='roles-label'
                        id='role_id'
                        value={roleId}
                        onChange={e => {
                          setRoleId(e.target.value)
                          setValue('role_id', e.target.value)
                        }}
                        MenuProps={MenuProps}
                        label='Roles'
                        defaultValue=''
                      >
                        {roles?.length > 0 ? (
                          roles?.map((item, index) => (
                            <MenuItem key={item?.id} value={item?.id}>
                              {item?.name}
                            </MenuItem>
                          ))
                        ) : (
                          <MenuItem selected={true} disabled>
                            No Role Found
                          </MenuItem>
                        )}
                      </Select>
                    ) : (
                      <CircularProgress size={30} />
                    )}

                    <FormHelperText>{errors['role_id']?.message}</FormHelperText>
                  </FormControl>
                </Box>
              )}
            </DialogContentText>
          </DialogContent>
          <DialogActions>
            <Button onClick={handleClose} color={'error'} variant={'outlined'} size={'small'}>
              Close
            </Button>
            <Button
              type={'submit'}
              disabled={buttonLoading}
              color={'success'}
              variant={'outlined'}
              size={'small'}
              endIcon={<SaveIcon className={'h-4'} />}
            >
              {buttonLoading ? <CircularProgress size={20} /> : 'Save'}
            </Button>
          </DialogActions>
        </form>
      </Dialog>
    </div>
  )
}

export default UserForm
